local f = CreateFrame("Frame", "JewelDailyQuestHelper")

local g = {
	["Bloodstone"] = 36917,
	["Huge Citrine"] = 36929,
	["Dark Jade"] = 36932,
	["Chalcedony"] = 36923,
	["Shadow Crystal"] = 36926,
	["Sun Crystal"] = 36920,
}

f.bankOpen = false
f.queuedItem = nil

f.quests = {
	[12962] = {g["Bloodstone"],g["Huge Citrine"]}, --Shipment: Bright Armor Relic
	[12958] = {g["Dark Jade"],g["Bloodstone"]}, --Shipment: Blood Jade Amulet
	[12959] = {g["Chalcedony"],g["Shadow Crystal"]}, --Shipment: Glowing Ivory Figurine
	[12961] = {g["Sun Crystal"],g["Dark Jade"]}, --Shipment: Intricate Bone Figurine
	[12963] = {g["Sun Crystal"],g["Shadow Crystal"]}, --Shipment: Shifting Sun Curio
	[12960] = {g["Huge Citrine"],g["Sun Crystal"]} --Shipment: Wicked Sun Brooch
}

f.reqItems = {
	--[itemID] = num,
}


function f:GetQuestID(questIndex)
	local questLink = GetQuestLink(questIndex)
	if not questLink then return nil end
	local questID = string.match(questLink, "Hquest:%d+")
	if questID then
		questID = string.gsub(questID,"Hquest:","")
		return tonumber(questID)
	end
end

f.events = {}

f.events.QUEST_ACCEPTED = function(self,questIndex)
	local questID = self:GetQuestID(questIndex)
	self:CheckItems(questID)
end


f.events.QUEST_LOG_UPDATE = function(self)
	self:UpdateQuests()
end


f.events.UNIT_QUEST_LOG_CHANGED = function(self,unit)
	if unit ~= "player" then return end
	self:UpdateQuests()
end


f.events.BAG_UPDATE = function(self, bagID)
	if not (bagID >= 0 and bagID <=4) then return end

	local queuedItem = self.queuedItem
	if queuedItem then
		local num = GetItemCount(queuedItem)
		if num == 0 then
			return
		else
			self.queuedItem = nil
		end
	end

	self:UpdateFreeSlots()
	self:UpdateQuests()
	self:TakeItemFromBank(next(self.reqItems))
end


f.events.BANKFRAME_OPENED = function(self)
	self.bankOpen = true
	self:TakeItemFromBank(next(self.reqItems))
end

f.events.BANKFRAME_CLOSED = function(self)
	self.bankOpen = false

end


function f:TakeItemFromBank(itemID,amount)
	if not self.bankOpen then return end
	if GetCursorInfo() then 
		self:RegisterEvent("CURSOR_UPDATE")
		return
	end
	local numBankSlots = GetNumBankSlots()

	for i=0,numBankSlots do
		local bagID = (i==0 and -1) or (NUM_BAG_SLOTS + i)
		for slot=1,GetContainerNumSlots(bagID) do
			local itemID = GetContainerItemID(bagID,slot)
			if self.queuedItem then return end
			if itemID and self.reqItems[itemID] and self.reqItems[itemID] > 0 then
				 if GetCursorInfo() then 
					self:RegisterEvent("CURSOR_UPDATE")
					return
				end
				local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bagID, slot)
				if itemCount <= amount then
					PickupContainerItem(bagID,slot)
					self:PutItemInBags()
					amount = amount - itemCount
				else
					if itemCount > amount then
						SplitContainerItem(bagID,slot,amount)
						self:PutItemInBags()
						amount = 0
					end
				end
				self.queuedItem = itemID
				self.reqItems[itemID] = amount
			end
		end
	end
end

function f:UpdateQuests()
	for k,v in pairs(self.reqItems) do
		self.reqItems[k] = nil
	end
	local numQ = GetNumQuestLogEntries()
	for questIndex=1,numQ do
		local questID = self:GetQuestID(questIndex)
		if questID then
			if self:CheckItems(questID) then
				return
			end
		end
	end
	--no quest found, clear reqItems table
end

function f:CheckItems(questID)
	local t = self.quests[questID]
	if not t then return end

	for k,v in pairs(self.reqItems) do
		self.reqItems[k] = nil
	end

	local ret = false

	for i=1,#t do
		local itemID = t[i]
		local num = GetItemCount(itemID)
		if num < 1 then
			self.reqItems[itemID] = 1
			ret = true
		end
	end
	return ret
end

f.freeSlots = {
	--[bagID] = num,
}
function f:PutItemInBags()
	for bagID = 0,4 do
		if self.freeSlots[bagID] and self.freeSlots[bagID] > 1 then
			if bagID == 0 then
				PutItemInBackpack()
			else
				PutItemInBag(ContainerIDToInventoryID(bagID))
			end
			self.freeSlots[bagID] = self.freeSlots[bagID] - 1
			break
		end
	end
end

function f:UpdateFreeSlots()
	for bagID=0,4 do
		local numFreeSlots, BagType = GetContainerNumFreeSlots(bagID)
		if BagType == 0 or BagType == 512 then
			self.freeSlots[bagID] = numFreeSlots
		end
	end
end


f.events.PLAYER_ENTERING_WORLD = function(self)
	self:UpdateFreeSlots()
	self:UpdateQuests()
end


f:SetScript("OnEvent", function(self, event, ...)
	self.events[event](self,...)
end)


for event,func in pairs(f.events) do
	f:RegisterEvent(event)
end


f.events.CURSOR_UPDATE = function(self)
	if GetCursorInfo() then return end
	self:UnregisterEvent("CURSOR_UPDATE")
	if self.bankOpen then
		self:TakeItemFromBank(next(self.reqItems))
	end
end

